/*
** Module   : NativeAPICaller.java
** Abstract : API for remote access to native library management and function invocation
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 GES 20130110 First version. The idea is to provide remote access from the server to the
**                  client-side for native library management and function invocation.
*/

package com.goldencode.p2j.library;

/**
 * This defines an API for native library management and function invocation.
 */
public interface NativeAPICaller
{
   /**
    * Invoke the defined native API call, loading (and optionally unloading) the library as
    * needed.
    * 
    * @param    libname
    *           The library name where this native procedure should be found.
    * @param    funcname
    *           The function name being called in the library (except where an ordinal is being
    *           used).
    * @param    ordinal
    *           The entry point's ordinal or -1 if the entry point should be found by name.
    * @param    persistent
    *           <code>true</code> to leave the library loaded when the native call is complete. 
    * @param    conv
    *           The calling convention to be used in this native call.
    * @param    signature
    *           Contains the return value and argument descriptors.  In the case of the arguments
    *           there will be 1 descriptor for each argument to the call (where the 0 index is
    *           the leftmost argument and the numargs - 1 index is the rightmost).  Each
    *           descriptor defines the type and will contain any input value in the case of an
    *           argument.  If the return value descriptor is <code>null</code>, the call should
    *           be treated as a void return.
    *
    * @return   The same signature instance will be updated with output (and/or return data)
    *           upon a successful call or with an error code on any failure.
    */
   public Signature invoke(String            libname,
                           String            funcname,
                           int               ordinal,
                           boolean           persistent,
                           CallingConvention conv,
                           Signature         signature);
   
   /**
    * Attempts to unload the library identified by the given name.
    *
    * @param    libname
    *           The name of the library to attempt to unload.
    */ 
   public void release(String libname);
}
